{ **************************************************************************** }
{ *                                                                          * }
{ * FOWLING (Elektronika) Simulator                                          * }
{ * by Luca "MADrigal" Antignano, October 2002 (revision: August 2021)       * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;       im_background_screen_top: TImage;

    btn_game_a_down:          TImage;       btn_game_a_top:           TImage;
    btn_game_b_down:          TImage;       btn_game_b_top:           TImage;
    btn_time_down:            TImage;       btn_time_top:             TImage;
    btn_acl_down:             TImage;       btn_acl_top:              TImage;
                                            btn_alarm_top:            TImage;

    btn_1_nw_down:            TImage;       btn_1_nw_top:             TImage;
    btn_2_ne_down:            TImage;       btn_2_ne_top:             TImage;
    btn_3_sw_down:            TImage;       btn_3_sw_top:             TImage;
    btn_4_se_down:            TImage;       btn_4_se_top:             TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_alarm_1: TImage; im_panel_alarm_2: TImage; im_panel_alarm_3: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_digit_1: TImage; im_panel_digit_2: TImage; im_panel_digit_3: TImage; im_panel_digit_4: TImage;

    im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage;

    im_panel_game_a: TImage; im_panel_game_b: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // on-screen items
    im_hero_w: TImage; im_hero_e: TImage;
    im_hero_nw: TImage; im_hero_ne: TImage; im_hero_sw: TImage; im_hero_se: TImage;

    im_enemy_nw1: TImage; im_enemy_nw2: TImage; im_enemy_nw3: TImage; im_enemy_nw4: TImage; im_enemy_nw5: TImage;
    im_enemy_ne1: TImage; im_enemy_ne2: TImage; im_enemy_ne3: TImage; im_enemy_ne4: TImage; im_enemy_ne5: TImage;
    im_enemy_sw1: TImage; im_enemy_sw2: TImage; im_enemy_sw3: TImage; im_enemy_sw4: TImage; im_enemy_sw5: TImage;
    im_enemy_se1: TImage; im_enemy_se2: TImage; im_enemy_se3: TImage; im_enemy_se4: TImage; im_enemy_se5: TImage;

    im_miss_w1: TImage; im_miss_w2: TImage; im_miss_w3: TImage; im_miss_w4: TImage; im_miss_w5: TImage;
    im_miss_e1: TImage; im_miss_e2: TImage; im_miss_e3: TImage; im_miss_e4: TImage; im_miss_e5: TImage;

{ * TIMERS                                                                   * }
    Timer_Chance:     TTimer;
    Timer_Clock:      TTimer;
    Timer_Game:       TTimer;
    Timer_Game_Over:  TTimer;
    Timer_Miss_Long:  TTimer;
    Timer_Miss_Short: TTimer;
    Timer_Point:      TTimer;
    Timer_Observer:   TTimer;
    Timer_Show_Miss:  TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_Hero                 (p: integer);

    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_1_nw_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_1_nw_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_2_ne_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_2_ne_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_3_sw_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_3_sw_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_4_se_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_4_se_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_1_nw_down;                procedure Inp_1_nw_up;
    procedure Inp_2_ne_down;                procedure Inp_2_ne_up;
    procedure Inp_3_sw_down;                procedure Inp_3_sw_up;
    procedure Inp_4_se_down;                procedure Inp_4_se_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Game_Start;
    procedure Gam_Set_Disabled_Enemy;
    procedure Gam_Set_Gameover_Sequence;
    procedure Gam_Set_Miss_Sequence;
    procedure Gam_Set_Mode;
    procedure Gam_Set_Timer_Game_Speed;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_ChanceTimer             (Sender: TObject);
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_Miss_LongTimer          (Sender: TObject);
    procedure Timer_Miss_ShortTimer         (Sender: TObject);
    procedure Timer_ObserverTimer           (Sender: TObject);
    procedure Timer_PointTimer              (Sender: TObject);
    procedure Timer_Show_MissTimer          (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Fowling (Elektronika)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     = 'Software\MADrigal\Egg-MG1850';                                 // registry key

  HINT_CLICK  = 'click or press';
  HINT_GAME_A = 'to start game A (hold to view highscore)';
  HINT_GAME_B = 'to start game B (hold to view highscore)';
  HINT_TIME   = 'to view current time (hold to view alarm time)';
  HINT_ACL    = 'to reset the game';
  HINT_ALARM  = 'button disabled';
  HINT_MOVE   = 'arrow keys';

{}MENU_ITEMS = 13;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_gameover,
  pfs_miss,
  pfs_point:               PFSoundSample;                                       // sound samples

  apfs_move:               array[1..4]       of PFSoundSample;                  // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_alarm:         array[1..3]       of TImage;                         // alarm panel
  aIm_panel_miss:          array[1..3]       of TImage;                         // miss marks panel
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score/time

  aIm_hero:                array[1..4]       of TImage;                         // game hero (1=NW, 2=NE, 3=SW, 4=SE)
  aIm_enemy:               array[1..4, 1..5] of TImage;                         // rows of enemies (1=NW, 2=NE, 3=SW, 4=SE)
  aIm_miss:                array[1..2, 1..5] of TImage;                         // miss animation (1=W, 2=E)

{ * arrays of mixed types * }
  aKeys:                   array[1..8] of record                                // keys management (values)
                                            int:  integer;
                                            word: word;
                                          end;

  aKeyHints:               array[1..40] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=ACL, 1=time, etc.
  iScore,                                                                       // 0-999
  iHiScoreA,                                                                    // 0-999
  iHiScoreB,                                                                    // 0-999
  iMiss,                                                                        // 0-7

  iHeroPosition,                                                                // position of game hero         (1=NW, 2=NE, 3=SW, 4=SE)
  iActiveEnemy,                                                                 // row of currently active enemy (1=NW, 2=NE, 3=SW, 4=SE)
  iInactiveEnemy,                                                               // row of inactive enemies       (1=NW, 2=NE, 3=SW, 4=SE)
  iMaxOnscreen                                                                  // maximum amount of enemies simultaneously on screen
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bLeftPress,                                                                   // true while key is pressed
  bRightPress,                                                                  // true while key is pressed
  bUpPress,                                                                     // true while key is pressed
  bDownPress,                                                                   // true while key is pressed
  bKeypressed,                                                                  // true while key is pressed
  bControls                                                                     // true while player can actually operate the main character
                         : boolean;

  // Game variables: loops
  iMissSide,
  iLoopMiss,
  iLoopChance
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel (alarm, time/score, game a/b, etc.)
  aIm_panel_digit_pos[1] := im_panel_digit_1; aIm_panel_digit_pos[2] := im_panel_digit_2; aIm_panel_digit_pos[3] := im_panel_digit_3; aIm_panel_digit_pos[4] := im_panel_digit_4;

  aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;

  aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2; aIm_panel_alarm[3] := im_panel_alarm_3; { aIm_panel_alarm[4] := im_panel_alarm_4; aIm_panel_alarm[5] := im_panel_alarm_5; }

  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4; aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  aIm_hero[1] := im_hero_nw; aIm_hero[2] := im_hero_ne; aIm_hero[3] := im_hero_sw; aIm_hero[4] := im_hero_se;

  aIm_enemy[1, 1] := im_enemy_nw1; aIm_enemy[1, 2] := im_enemy_nw2; aIm_enemy[1, 3] := im_enemy_nw3; aIm_enemy[1, 4] := im_enemy_nw4; aIm_enemy[1, 5] := im_enemy_nw5;
  aIm_enemy[2, 1] := im_enemy_ne1; aIm_enemy[2, 2] := im_enemy_ne2; aIm_enemy[2, 3] := im_enemy_ne3; aIm_enemy[2, 4] := im_enemy_ne4; aIm_enemy[2, 5] := im_enemy_ne5;
  aIm_enemy[3, 1] := im_enemy_sw1; aIm_enemy[3, 2] := im_enemy_sw2; aIm_enemy[3, 3] := im_enemy_sw3; aIm_enemy[3, 4] := im_enemy_sw4; aIm_enemy[3, 5] := im_enemy_sw5;
  aIm_enemy[4, 1] := im_enemy_se1; aIm_enemy[4, 2] := im_enemy_se2; aIm_enemy[4, 3] := im_enemy_se3; aIm_enemy[4, 4] := im_enemy_se4; aIm_enemy[4, 5] := im_enemy_se5;

  aIm_miss[1, 1] := im_miss_w1; aIm_miss[1, 2] := im_miss_w2; aIm_miss[1, 3] := im_miss_w3; aIm_miss[1, 4] := im_miss_w4; aIm_miss[1, 5] := im_miss_w5; { aIm_miss[1, 6] := im_miss_w6; }
  aIm_miss[2, 1] := im_miss_e1; aIm_miss[2, 2] := im_miss_e2; aIm_miss[2, 3] := im_miss_e3; aIm_miss[2, 4] := im_miss_e4; aIm_miss[2, 5] := im_miss_e5; { aIm_miss[2, 6] := im_miss_e6; }

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''alt'' key';
  aKeyHints[38] := '''ctrl'' key';
  aKeyHints[39] := '''shift'' key';
  aKeyHints[40] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 27);                             // "(1) NW" (default: "Q")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 26);                             // "(2) NE" (default: "P")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 11);                             // "(3) SW" (default: "A")
  aKeys[8].int := iReg.ReadInteger('', 'Key8', 22);                             // "(4) SE" (default: "L")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 8 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A;  // "Game A"
  btn_game_b_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B;  // "Game B"
  btn_time_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;    // "Time"
  btn_acl_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;     // "ACL"
  btn_alarm_top.Hint  := HINT_ALARM;                                                      // "Alarm"
  btn_1_nw_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int] + ' or ' + HINT_MOVE; // "(1) NW"
  btn_2_ne_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int] + ' or ' + HINT_MOVE; // "(2) NE"
  btn_3_sw_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int] + ' or ' + HINT_MOVE; // "(3) SW"
  btn_4_se_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[8].int] + ' or ' + HINT_MOVE; // "(4) SE"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bUpPress    := false;                                                         // no key is currently being kept pressed
  bDownPress  := false;                                                         // no key is currently being kept pressed
  bLeftPress  := false;                                                         // no key is currently being kept pressed
  bRightPress := false;                                                         // no key is currently being kept pressed
  bKeypressed := false;                                                         // no key is currently being kept pressed
  bControls   := false;                                                         // prevents main character movement

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
  var
    i: integer;
begin
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_miss);
  FSOUND_Sample_Free(pfs_point);

  for i := 1 to 4 do
    FSOUND_Sample_Free(apfs_move[i]);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i, j: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;

  { im_panel_time_am.Visible := b; }
  im_panel_time_pm.Visible := false;
  im_panel_time_colon.Visible := b;

  im_panel_game_a.Visible := b;
  im_panel_game_b.Visible := b;

  for i := 1 to 3 do
    aIm_panel_alarm[i].Visible := b;

  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := b;

  im_hero_w.Visible := b;
  im_hero_e.Visible := b;

  for i := 1 to 4 do
    begin
      aIm_hero[i].Visible := b;

      for j := 1 to 5 do
        aIm_enemy[i, j].Visible := b;
    end;

  for i := 1 to 2 do
    for j := 1 to 5 do
      aIm_miss[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_Hero(p: integer);
{ displays/hides main character in 'p' position,                               }
  var
    i: integer;
begin
  if p = 0 then
    begin
      for i := 1 to 4 do
        aIm_hero[i].Visible := false;

      im_hero_w.Visible := false;
      im_hero_e.Visible := false;
    end
  else
    begin
      for i := 1 to 4 do
        aIm_hero[i].Visible := i = p;

      im_hero_w.Visible := Odd(p);
      im_hero_e.Visible := not Odd(p);
    end;
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 1000 do                                                            // displays "1000" as "...0"
    Dec(t, 1000);

  aIm_panel_digit_pos[1].Picture := nil;                                        // thousands (always hidden)

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      if t > 99 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div 100 mod 10].Picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t >  9 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div  10 mod 10].Picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                     aIm_panel_digit_pos[4].Picture := aIm_panel_number[t         mod 10].Picture                                             // units
    end
  else                                                                            // hide
    for i := 2 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_1_nw_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ NW button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_1_nw_down;
end;





procedure TForm1.btn_1_nw_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ NW button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_1_nw_up;
end;





procedure TForm1.btn_2_ne_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SW button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_2_ne_down;
end;





procedure TForm1.btn_2_ne_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SW button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_2_ne_up;
end;





procedure TForm1.btn_3_sw_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ NE button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_3_sw_down;
end;





procedure TForm1.btn_3_sw_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ NE button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_3_sw_up;
end;





procedure TForm1.btn_4_se_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SE button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_4_se_down;
end;





procedure TForm1.btn_4_se_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SE button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_4_se_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  case Key of
    VK_UP:    bUpPress    := true;                                              // arrow up
    VK_DOWN:  bDownPress  := true;                                              // arrow down
    VK_LEFT:  bLeftPress  := true;                                              // arrow left
    VK_RIGHT: bRightPress := true;                                              // arrow right
  end; // case..of

  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                                     // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                                     // Game B
  if Key = aKeys[3].word then Inp_time_down   else                                     // Time
  if Key = aKeys[4].word then Inp_acl_down    else                                     // ACL
  if (Key = aKeys[5].word) or ((bLeftPress)  and (bUpPress))   then Inp_1_nw_down else // up-left
  if (Key = aKeys[6].word) or ((bRightPress) and (bUpPress))   then Inp_2_ne_down else // up-right
  if (Key = aKeys[7].word) or ((bLeftPress)  and (bDownPress)) then Inp_3_sw_down else // down-left
  if (Key = aKeys[8].word) or ((bRightPress) and (bDownPress)) then Inp_4_se_down;     // down-right
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  case Key of
    VK_UP:    begin bUpPress    := false; btn_1_nw_down.Visible := false; btn_2_ne_down.Visible := false; bKeypressed := false; end; // arrow up
    VK_DOWN:  begin bDownPress  := false; btn_3_sw_down.Visible := false; btn_4_se_down.Visible := false; bKeypressed := false; end; // arrow down
    VK_LEFT:  begin bLeftPress  := false; btn_1_nw_down.Visible := false; btn_3_sw_down.Visible := false; bKeypressed := false; end; // arrow left
    VK_RIGHT: begin bRightPress := false; btn_2_ne_down.Visible := false; btn_4_se_down.Visible := false; bKeypressed := false; end; // arrow right
  end; // case..of

  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_acl_up    else                                // ACL
  if Key = aKeys[5].word then Inp_1_nw_up   else                                // up-left
  if Key = aKeys[6].word then Inp_2_ne_up   else                                // up-right
  if Key = aKeys[7].word then Inp_3_sw_up   else                                // down-left
  if Key = aKeys[8].word then Inp_4_se_up;                                      // down-right
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      // sets hero to a random position if "game A" is pressed when in "ACL/2" mode
      if iMode = 1 then
        begin
          iHeroPosition := Random(4) + 1;
          Vid_Show_Hero(iHeroPosition);
        end;

      iMode := 4; Gam_Set_Mode;                                                   //         -> hiscore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 4 then                                                             // hiscore A ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      // sets hero to a random position if "game A" is pressed when in "ACL/2" mode
      if iMode = 1 then
        begin
          iHeroPosition := Random(4) + 1;
          Vid_Show_Hero(iHeroPosition);
        end;

      iMode := 6; Gam_Set_Mode;                                                   //         -> hiscore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 6 then                                                             // hiscore B ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> game B
    end;
end;





procedure TForm1.Inp_time_down;
{ time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //         -> alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 2 then                                                             // alarm ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //     -> time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  iMode := 0; Gam_Set_Mode;                                                     // any mode -> ACL/1
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  iMode := 1; Gam_Set_Mode;                                                     // ACL/1  -> ACL/2
end;





procedure TForm1.Inp_1_nw_down;
{ NW button: PRESS                                                             }
begin
  btn_1_nw_down.Visible := true;

  if ( (iMode = 5) or (iMode = 7) or (iMode = 8) ) and (not bKeypressed) then
    begin
      bKeypressed := true;

      if bControls then
        begin
          iHeroPosition := 1;
          Vid_Show_Hero(iHeroPosition);
        end;
    end;
end;





procedure TForm1.Inp_1_nw_up;
{ NW button: RELEASE                                                           }
begin
  btn_1_nw_down.Visible := false;

  bKeypressed := false;
end;





procedure TForm1.Inp_2_ne_down;
{ NE button: PRESS                                                             }
begin
  btn_2_ne_down.Visible := true;

  if ( (iMode = 5) or (iMode = 7) or (iMode = 8) ) and (not bKeypressed) then
    begin
      bKeypressed := true;

      if bControls then
        begin
          iHeroPosition := 2;
          Vid_Show_Hero(iHeroPosition);
        end;
    end;
end;





procedure TForm1.Inp_2_ne_up;
{ NE button: RELEASE                                                           }
begin
  btn_2_ne_down.Visible := false;

  bKeypressed := false;
end;





procedure TForm1.Inp_3_sw_down;
{ SW button: PRESS                                                             }
begin
  btn_3_sw_down.Visible := true;

  if ( (iMode = 5) or (iMode = 7) or (iMode = 8) ) and (not bKeypressed) then
    begin
      bKeypressed := true;

      if bControls then
        begin
          iHeroPosition := 3;
          Vid_Show_Hero(iHeroPosition);
        end;
    end;
end;





procedure TForm1.Inp_3_sw_up;
{ NW button: RELEASE                                                           }
begin
  btn_3_sw_down.Visible := false;

  bKeypressed := false;
end;





procedure TForm1.Inp_4_se_down;
{ SE button: PRESS                                                             }
begin
  btn_4_se_down.Visible := true;

  if ( (iMode = 5) or (iMode = 7) or (iMode = 8) ) and (not bKeypressed) then
    begin
      bKeypressed := true;

      if bControls then
        begin
          iHeroPosition := 4;
          Vid_Show_Hero(iHeroPosition);
        end;
    end;
end;





procedure TForm1.Inp_4_se_up;
{ NW button: RELEASE                                                           }
begin
  btn_4_se_down.Visible := false;

  bKeypressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
begin
  Timer_Chance.Enabled     := false;
  Timer_Clock.Enabled      := false;
  Timer_Game.Enabled       := false;
  Timer_Game_Over.Enabled  := false;
  Timer_Miss_Long.Enabled  := false;
  Timer_Miss_Short.Enabled := false;
  Timer_Point.Enabled      := false;
  Timer_Observer.Enabled   := false;
  Timer_Show_Miss.Enabled  := false;
end;





procedure TForm1.Gam_Game_Start;
{ initializes and starts new game                                              }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  // randomize "observer" on screen at game start
  present := Now;
  DecodeTime(present, hour, min, sec, msec);
  aIm_panel_alarm[1].Visible := Odd(sec div 5);

  Timer_Observer.Interval := 100;
  Timer_Observer.Enabled := true;

  // various settings
  iMiss := 0;
  iScore := 0;
  Vid_Show_Panel_Score(iScore, true);

  bKeypressed := false;
  iInactiveEnemy := 0;                                                          // needed by Gam_Set_Disabled_Enemy

  if iMode = 5 then
    Gam_Set_Disabled_Enemy;                                                     // limits active rows of enemies to 3 in game A

  repeat                                                                        // randomizes active row of enemies
    iActiveEnemy := Random(4) + 1
  until
    iActiveEnemy <> iInactiveEnemy;

  bControls := true;

  Gam_Set_Timer_Game_Speed;                                                     // game timer starts ticking
  Timer_Game.Enabled := true;
end;






procedure TForm1.Gam_Set_Disabled_Enemy;
{ sets a random row out of 4 which is unused in game A                         }
  var
    i: integer;
begin
  repeat
    i := Random(4) + 1
  until
    i <> iInactiveEnemy;                                                        // needed to prevent that iInactiveEnemy gets the same value twice

  iInactiveEnemy := i;
end;





procedure TForm1.Gam_Set_Gameover_Sequence;
{ initializes and starts "game over" routine                                   }
begin
  Snd_Play(pfs_gameover);

  if (iMode = 5) and (iScore > iHiScoreA) then
    begin
      iHiScoreA := iScore;
      iReg.WriteInteger('', 'HiScore1', iHiScoreA);
    end;

  if (iMode = 7) and (iScore > iHiScoreB) then
    begin
      iHiScoreB := iScore;
      iReg.WriteInteger('', 'HiScore2', iHiScoreB);
    end;

  iMode := 8; Gam_Set_Mode;

  Timer_Game_Over.Interval := 300000;
  Timer_Game_Over.Enabled := true;
end;





procedure TForm1.Gam_Set_Miss_Sequence;
{ initializes and starts "miss" routines                                       }
begin
  if Odd(iActiveEnemy) then
    iMissSide := 1                                                              // miss occured on LEFT side
  else
    iMissSide := 2;                                                             // miss occured on RIGHT side

  iLoopMiss := 1;

  if aIm_panel_alarm[1].Visible then
    begin
      Timer_Miss_Long.Interval := 50;
      Timer_Miss_Long.Enabled := true
    end
  else
    begin
      Timer_Miss_Short.Enabled := true;
      Timer_Miss_Short.Interval := 50;
    end;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/ACL mode, depending on the value of the iMode variable        }
begin
  case iMode of
    0: begin                                                                    // ACL 1 (ACL button press)
         Gam_Stop_All_Timers;
         Snd_Silence;

         bControls := false;
       end;
    1: Vid_Show_All(true);                                                      // ACL 2 (ACL button release)
    2: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         aIm_panel_alarm[1].Visible := true;
         aIm_panel_alarm[2].Visible := true;

         im_panel_time_pm.Visible := true;
         im_panel_time_colon.Visible := true;

         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
         aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
         aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;

         bControls := false;
       end;
    3: begin                                                                    // TIME (Time button release)
         Vid_Show_All(false);

         aIm_panel_alarm[1].Visible := false;
         aIm_panel_alarm[2].Visible := false;

         im_panel_time_colon.Visible := true;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;

         bControls := false;
       end;
    4, 6: begin                                                                 // HISCORE A/B (Game A/B button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         Vid_Show_Hero(iHeroPosition);

         aIm_panel_alarm[1].Visible := true;

         case iMode of
           4: begin
                im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true);
              end;
           6: begin
                im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true);
              end;
         end; // case..of

         bControls := false;
       end;
    5, 7: Gam_Game_Start;                                                       // GAME A/B (Game A/B button release)
    8: begin                                                                    // GAME OVER
         // do nothing
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Timer_Game_Speed;
{ sets enemy speed                                                             }
  var
    i, j,
    iTempScore,
    iTempCurrentRow: integer;
    bTempCurrentRow: boolean;
begin
  iTempScore := iScore;

  while iTempScore > 99 do
   dec(iTempScore, 100);

  // max enemies on screen
  case iScore of
       0..99: case iTempScore of
                  0..4: iMaxOnscreen := 1;
                  5..9: iMaxOnscreen := 2;
                10..49: iMaxOnscreen := 3;
                50..89: iMaxOnscreen := 4;
                90..99: iMaxOnscreen := 5;
              end; // case..of
    100..199: case iTempScore of
                 0..39: iMaxOnscreen := 3;
                40..79: iMaxOnscreen := 4;
                80..99: iMaxOnscreen := 5;
               end; // case..of
    200..299: case iTempScore of
                 0..29: iMaxOnscreen := 3;
                30..79: iMaxOnscreen := 4;
                80..99: iMaxOnscreen := 5;
               end; // case..of
    else      case iTempScore of
                 0..29: iMaxOnscreen := 3;
                30..69: iMaxOnscreen := 4;
                70..99: iMaxOnscreen := 5;
              end; // case..of
  end; // case..of

  // game speed
  case iMode of
    5: case iTempScore of                                                       // game A
           0..4: Timer_Game.Interval := 390; (*verified*) // doubles automatically!
           5..9: Timer_Game.Interval := 400; (*verified*)
         10..19: Timer_Game.Interval := 380; (*verified*)
         20..29: Timer_Game.Interval := 370; (*verified*)
         30..39: Timer_Game.Interval := 365; (*verified*)
         40..49: Timer_Game.Interval := 355; (*verified*)
         50..59: Timer_Game.Interval := 345; (*verified*)
         60..69: Timer_Game.Interval := 335; (*verified*)
         70..79: Timer_Game.Interval := 325; (*assumed*)
         80..89: Timer_Game.Interval := 310; (*assumed*)
         90..99: Timer_Game.Interval := 290; (*assumed*)
       end; // case..of
    7: case iTempScore of                                                       // game B
           0..4: Timer_Game.Interval := 325; (*verified*) // doubles automatically!
           5..9: Timer_Game.Interval := 330; (*verified*)
         10..19: Timer_Game.Interval := 320; (*verified*)
         20..29: Timer_Game.Interval := 300; (*verified*)
         30..39: Timer_Game.Interval := 290; (*verified*)
         40..49: Timer_Game.Interval := 280; (*assumed*)
         50..59: Timer_Game.Interval := 270; (*assumed*)
         60..69: Timer_Game.Interval := 260; (*assumed*)
         70..79: Timer_Game.Interval := 250; (*assumed*)
         80..89: Timer_Game.Interval := 240; (*assumed*)
         90..99: Timer_Game.Interval := 230; (*assumed*)
       end; // case..of
  end; // case..of

  if iScore > 99 then
    Timer_Game.Interval := (Timer_Game.Interval - 60);                          // increases speed a bit if score equals 100 or above

  // doubles timer speed or not?
  iTempCurrentRow := 0;

  for i := 1 to 4 do
    begin
      bTempCurrentRow := false;

      for j := 1 to 5 do
        if aIm_enemy[i,j].Visible then
          bTempCurrentRow := true;

      if bTempCurrentRow
        then Inc(iTempCurrentRow);
    end;

  if iTempCurrentRow = 1
    then Timer_Game.Interval := (Timer_Game.Interval) * 2;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_ChanceTimer(Sender: TObject);
{ enters "chance time" mode                                                    }
  var
    i: integer;
begin
  case iLoopChance of
    1: Snd_Play(pfs_miss);
    2, 4, 6, 8, 10, 12: for i := 1 to 3 do aIm_panel_miss[i].Visible := false;
    3, 5, 7, 9, 11, 13: begin Snd_Play(pfs_miss); Timer_Show_MissTimer(Sender); end;
   14: begin
         iMiss := 0;
         Timer_Show_MissTimer(Sender);

         bControls := true;

         Timer_Observer.Interval := 100;
         Timer_Observer.Enabled := true;

         Timer_Game.Enabled := true;

         Timer_Chance.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopChance);
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      { im_panel_time_am.Visible := false; }
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      { im_panel_time_am.Visible := true; }
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;

  // demo mode animation
  iHeroPosition := Random(4) + 1;
  Vid_Show_Hero(iHeroPosition);
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i, j,
    iTempOnscreen: integer;

    bTempSound,
    bTempGenerate,
    bTempMiss:     boolean;

    bUsedRow,
    bRowHasEnemy:  boolean;
begin
  bTempMiss := false;

  if iActiveEnemy = iInactiveEnemy then
    begin
      Inc(iActiveEnemy);

      if iActiveEnemy > 4 then                                                  // blocks "disabled" enemy in game A mode
        iActiveEnemy := 1;
    end;

  // is there any enemy on current row?
  bUsedRow := false;
  bRowHasEnemy := false;

  for i := 1 to 5 do
    if aIm_enemy[iActiveEnemy, i].Visible then
      bRowHasEnemy := true;

  if bRowHasEnemy then
    begin
      bUsedRow := true;

      // checks for possible miss/catch
      if aIm_enemy[iActiveEnemy, 5].Visible then
        begin
          Snd_Play(apfs_move[iActiveEnemy]);

          if iHeroPosition = iActiveEnemy then                                  // enemy caught: increase score, etc.
            begin
              aIm_enemy[iActiveEnemy, 5].Visible := false;

              Timer_Point.Interval := 120;
              Timer_Point.Enabled := true;

              Timer_Game.Enabled := false;
            end
          else                                                                  // enemy not caught: execute "miss" procedure
            begin
              bTempMiss := true;
              Gam_Set_Miss_Sequence;

              Timer_Game.Enabled := false;
            end;
        end
      else                                                                      // no miss/catch: move/generate enemies
        begin
          // enemy movement
          for i := 4 downto 1 do
            aIm_enemy[iActiveEnemy, i + 1].Visible := aIm_enemy[iActiveEnemy, i].Visible;

          // enemy generation
          bTempGenerate := Random(4) = 0;                                       // randomized generation (25%)

          if aIm_enemy[iActiveEnemy, 2].Visible                                 // no generation if enemy on position 2 of current row
            then bTempGenerate := false;

          case iActiveEnemy of                                                  // no generation if enemy was generated on previous row
            1: if aIm_enemy[4, 1].Visible then bTempGenerate := false;
            2: if aIm_enemy[1, 1].Visible then bTempGenerate := false;
            3: if aIm_enemy[2, 1].Visible then bTempGenerate := false;
            4: if aIm_enemy[3, 1].Visible then bTempGenerate := false;
          end;

          iTempOnscreen := 0;                                                   // counts enemies currently on screen
          for i := 1 to 4 do
            for j := 1 to 5 do
              if aIm_enemy[i, j].Visible then
                Inc(iTempOnscreen);

          if iTempOnscreen >= iMaxOnscreen                                      // no generation if there's too many enemies on screen
             then bTempGenerate := false;

          if iTempOnscreen = 0 then                                             // force generation if there's no enemies on screen
            bTempGenerate := true;

          aIm_enemy[iActiveEnemy,1].Visible := bTempGenerate;                   // actual generation of new enemy
        end;
    end
  else
    // generates new enemy and makes next column active
    begin
      bTempGenerate := Random(2) = 0;                                           // randomized generation (25%)

      if aIm_enemy[iActiveEnemy,2].Visible then                                 // no generation if enemy on position 2 of current row
        bTempGenerate := false;

      case iActiveEnemy of                                                      // no generation if enemy was generated on previous row
        1: if aIm_enemy[4, 1].Visible then bTempGenerate := false;
        2: if aIm_enemy[1, 1].Visible then bTempGenerate := false;
        3: if aIm_enemy[2, 1].Visible then bTempGenerate := false;
        4: if aIm_enemy[3, 1].Visible then bTempGenerate := false;
      end;

      iTempOnscreen := 0;                                                       // counts enemies currently on screen
      for i := 1 to 4 do
        for j := 1 to 5 do
          if aIm_enemy[i, j].Visible then
            Inc(iTempOnscreen);

       if iTempOnscreen >= iMaxOnscreen then                                    // no generation if there's too many enemies on screen
         bTempGenerate := false;

       if iTempOnscreen = 0 then                                                // force generation if there's no enemies on screen
         begin
           bTempGenerate := true;

           repeat                                                               // new enemy appears on a random row
             iActiveEnemy := Random(4) + 1;
           until
             iActiveEnemy <> iInactiveEnemy;
         end;

       if bTempGenerate then                                                    // actual generation of new enemy
         begin
           aIm_enemy[iActiveEnemy, 1].Visible := true;
           bUsedRow := true;
         end;
    end;

  if bUsedRow then
    begin
      bTempSound := false;                                                        // play sound sample or not play?

      for i := 1 to 5 do
        if aIm_enemy[iActiveEnemy,i].Visible then
          bTempSound := true;

       if bTempSound then
         Snd_Play(apfs_move[iActiveEnemy]);

       if not bTempMiss then                                                    // which row of enemies to move next?
         Inc(iActiveEnemy);

       if iActiveEnemy > 4 then
         iActiveEnemy := 1;

       Gam_Set_Timer_Game_Speed;
    end
  else
    begin
      Inc(iActiveEnemy);

      if iActiveEnemy > 4 then
        iActiveEnemy := 1;

      if iActiveEnemy = iInactiveEnemy then
        Inc(iActiveEnemy);

      if iActiveEnemy > 4 then
        iActiveEnemy := 1;

      Timer_GameTimer(sender);
    end;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ mode goes back into "time", five minutes after game is over                  }
begin
  iMode := 3; Gam_Set_Mode;

  Timer_Game_Over.Enabled := false;
end;





procedure TForm1.Timer_Miss_LongTimer(Sender: TObject);
{ long miss (worth half miss mark)                                             }
  var
    i, j: integer;
begin
  case iLoopMiss of
    1: begin
         for j := 1 to 4 do
           for i := 1 to 5 do
             aIm_enemy[j, i].Visible := false;

         Snd_Play(pfs_miss);         aIm_miss[iMissSide, 1].Visible := true;  aIm_miss[iMissSide, 2].Visible := true;  Timer_Miss_Long.Interval := 400;
       end;
    2: begin Snd_Play(apfs_move[2]); aIm_miss[iMissSide, 1].Visible := false; aIm_miss[iMissSide, 3].Visible := true;  Timer_Miss_Long.Interval := 560; end;
    3: begin Snd_Play(apfs_move[2]); aIm_miss[iMissSide, 3].Visible := false; aIm_miss[iMissSide, 4].Visible := true;  end;
    4: begin Snd_Play(apfs_move[2]); aIm_miss[iMissSide, 4].Visible := false; aIm_miss[iMissSide, 5].Visible := true;  end;
    5: begin Snd_Play(apfs_move[2]); aIm_miss[iMissSide, 5].Visible := false; { aIm_miss[iMissSide, 6].Visible := true; } Timer_Miss_Long.Interval := 700; end;
    6: begin                         { aIm_miss[iMissSide, 6].Visible := false; } aIm_miss[iMissSide, 2].Visible := false; end;
    7: begin
         Inc(iMiss);                                                            // misses increase by 1. Maximum misses is 6

         Timer_Show_Miss.Interval := 1;
         Timer_Show_Miss.Enabled := true;
       end;
    8: begin
         if iMiss < 6 then
           begin
             if iMode = 5 then
               Gam_Set_Disabled_Enemy;

             Timer_Observer.Interval := 100;
             Timer_Observer.Enabled := true;

             Timer_Game.Enabled := true;
           end
         else
           Gam_Set_Gameover_Sequence;

         Timer_Miss_Long.Enabled := false;
       end;
  end;

  Inc(iLoopMiss);
end;





procedure TForm1.Timer_Miss_ShortTimer(Sender: TObject);
{ short miss (worth 1 miss mark)                                               }
  var
    i, j: integer;
begin
  case iLoopMiss of
    1: begin
         for j := 1 to 4 do
           for i := 1 to 5 do
             aIm_enemy[j, i].Visible := false;

         Snd_Play(pfs_miss);

         aIm_miss[iMissSide, 1].Visible := true;
         aIm_miss[iMissSide, 2].Visible := true;

         Timer_Miss_Short.Interval := 2000;
       end;
    2: begin
         aIm_miss[iMissSide, 1].Visible := false;
         aIm_miss[iMissSide, 2].Visible := false;

         Inc(iMiss, 2);                                                         // misses increase by 2. Maximum misses is 6

         Timer_Show_Miss.Interval := 1;
         Timer_Show_Miss.Enabled := true;

         Timer_Miss_Short.Interval := 700;
       end;
    3: begin
         if iMiss < 6 then
           begin
             if iMode = 5 then
               Gam_Set_Disabled_Enemy;

             Timer_Observer.Interval := 120;
             Timer_Observer.Enabled := true;

             Timer_Game.Enabled := true;
           end
         else
           Gam_Set_Gameover_Sequence;

         Timer_Miss_Short.Enabled := false;
       end;
  end;

  Inc(iLoopMiss);
end;





procedure TForm1.Timer_ObserverTimer(Sender: TObject);
{ the "observer" (alarm character) blinks at every 5 seconds                   }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  aIm_panel_alarm[1].Visible := Odd(sec div 5);
end;





procedure TForm1.Timer_PointTimer(Sender: TObject);
{ player scored points                                                         }
begin
  Timer_Game.Enabled := true;

  if iMiss > 0 then
    begin
      Timer_Show_Miss.Interval := 1;
      Timer_Show_Miss.Enabled := true;
    end;

  Snd_Play(pfs_point);
  Inc(iScore);
  Vid_Show_Panel_Score(iScore, true);

  // checks for "chance time"
  if (iScore = 200) or (iScore = 500) then
    if iMiss <> 0 then
      begin
        Gam_Stop_All_Timers;
        Snd_Silence;

        bControls := false;

        iLoopChance := 1;
        Timer_Chance.Interval := 350;
        Timer_Chance.Enabled := true;
      end;

  Timer_Point.Enabled := false;
end;





procedure TForm1.Timer_Show_MissTimer(Sender: TObject);
{ displays misses panel (blinking)                                             }
  var
    i: integer;
begin
  case iMiss of
    0:    for i := 1 to 3 do
            aIm_panel_miss[i].Visible := false;
    1:    aIm_panel_miss[1].Visible := not aIm_panel_miss[1].Visible;
    2:    begin aIm_panel_miss[1].Visible := true; Timer_Show_Miss.Enabled := false; end;
    3:    begin aIm_panel_miss[1].Visible := true; aIm_panel_miss[2].Visible := not aIm_panel_miss[2].Visible; end;
    4:    begin aIm_panel_miss[1].Visible := true; aIm_panel_miss[2].Visible := true; Timer_Show_Miss.Enabled := false; end;
    5:    begin aIm_panel_miss[1].Visible := true; aIm_panel_miss[2].Visible := true; aIm_panel_miss[3].Visible := not aIm_panel_miss[3].Visible; end;
    6, 7: begin aIm_panel_miss[1].Visible := true; aIm_panel_miss[2].Visible := true; aIm_panel_miss[3].Visible := true; Timer_Show_Miss.Enabled := false; end;
  end;

  Timer_Show_Miss.Interval := 800;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
